import gettext
import locale
import os, gi, re, configparser, subprocess

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk, Gdk

t = gettext.translation('share_directory', "/usr/share/locale/")
t.install()
_ = t.gettext

log_actions_available = [
    ["aio_force", False], ["audit_file", False], ["brl_lock_windows", False], ["brl_unlock_windows", False],
    ["chdir", False], ["chflags", False], ["chmod", False], ["close", False], ["closedir", False],
    ["connect", False], ["connectpath", False], ["create_dfs_pathat", False], ["create_file", False],
    ["disconnect", False], ["disk_free", False], ["durable_cookie", False], ["durable_disconnect", False],
    ["durable_reconnect", False], ["fallocate", False], ["fchmod", False], ["fchown", False], ["fdopendir", False],
    ["fget_compression", False], ["fget_dos_attributes", False], ["fget_nt_acl_at", False], ["fgetxattr", False],
    ["file_id_create", False], ["flistxattr", False], ["fremovexattr", False], ["fs_capabilities", False],
    ["fsctl", False], ["fset_dos_attributes", False], ["fset_nt_acl", False], ["fsetxattr", False],
    ["fs_file_id", False], ["fstat", False], ["fsync", False], ["fsync_recv", False], ["fsync_send", False],
    ["ftruncate", False], ["get_alloc_size", False], ["get_dfs_referrals", False], ["get_dos_attributes", False],
    ["get_dos_attributes_recv", False], ["get_dos_attributes_send", False], ["getlock", False], ["get_nt_acl", False],
    ["get_quota", False], ["get_real_filename", False], ["get_shadow_copy_data", False], ["getwd", False],
    ["getxattr", False], ["getxattrat_recv", False], ["getxattrat_send", False], ["is_offline", False],
    ["kernel_flock", False], ["lchown", False], ["linkat", False], ["linux_setlease", False], ["listxattr", False],
    ["lock", True], ["lseek", False], ["lstat", False], ["mkdir", True], ["mkdirat", False], ["mknodat", False],
    ["ntimes", False], ["offload_read_recv", False], ["offload_read_send", False], ["offload_write_recv", False],
    ["offload_write_send", False], ["open", True], ["pread", False], ["pread_recv", False], ["pread_send", False],
    ["pwrite", True], ["pwrite_recv", False], ["pwrite_send", False], ["read", False], ["readdir", False],
    ["readdir_attr", False], ["readlinkat", False], ["realpath", False], ["recvfile", False], ["removexattr", False],
    ["rename", True], ["renameat", False], ["rewinddir", False], ["rmdir", True], ["seekdir", False],
    ["sendfile", True], ["set_compression", False], ["set_dos_attributes", False], ["set_offline", False],
    ["set_quota", False], ["setxattr", False], ["snap_check_path", False], ["snap_create", False],
    ["snap_delete", False], ["stat", False], ["statvfs", False], ["streaminfo", False], ["strict_lock_check", False],
    ["symlinkat", False], ["sys_acl_blob_get_fd", False], ["sys_acl_blob_get_file", False], ["sys_acl_get_fd", False],
    ["sys_acl_get_file", False], ["sys_acl_set_fd", False], ["telldir", False], ["translate_name", False],
    ["unlink", True], ["unlinkat", False], ["write", True]
]

log_prefixes_available = [
    ["%U", False], ["%G", False], ["%h", False], ["%L", False], ["%M", False], ["%R", False], ["%d", False],
    ["%a", False],
    ["%I", True], ["%m", True], ["%J", False], ["%i", False], ["%j", False], ["%T", False], ["%t", False],
    ["%D", False], ["%w", False], ["%$(envvar)", False], ["%S", True], ["%P", False], ["%u", True], ["%g", False],
    ["%H", False], ["%N", False]]

def get_domain_type():
    output = os.popen("realm list").readlines()
    realms = []
    for line in output:
        if 'server-software:' in line:
            realms.append(line[19:].strip())
    return realms


def get_domain_name():
    output = os.popen("hostname -d").readlines()[0].strip()
    return output

def get_ipa_domain_name():
    output = os.popen("hostname -d").readlines()[0].strip()
    return output


def get_smbd_LIBACL():
    output = os.popen("smbd -b | grep HAVE_LIBACL").readlines()[0]
    output = output.strip()
    if output == "HAVE_LIBACL":
        return True
    else:
        return False


def get_section(path, section):  # проверка существования секции
    config = configparser.ConfigParser(delimiters='=')
    config.read(path)
    if section in config.sections():
        return True
    else:
        return False

def add_new_section(path, section):  # добавление новой секции
    config = configparser.ConfigParser(delimiters='=')
    config.read(path)
    config.add_section(section)
    with open(path, 'w') as configfile:
        config.write(configfile)

def remove_old_section(path, section):  # удаление секции
    config = configparser.ConfigParser(delimiters='=')
    config.read(path)
    config.remove_section(section)
    with open(path, 'w') as configfile:
        config.write(configfile)

def set_artibute(path, section, setting, value):  # добавление или изменение параметров в конфигах
    config = configparser.ConfigParser(delimiters='=')
    flag_ex = False
    while not flag_ex:
        try:
            config.read(path)
            flag_ex = True
        except configparser.DuplicateOptionError as e:
            with open(path, "r") as config_file:
                new_file = config_file.readlines()
            for line in new_file:
                if f"{e.option} = " in line:
                    new_file.remove(line)
                    break
            with open(path, "w") as config_file:
                config_file.writelines(new_file)
    config[section][setting] = value
    with open(path, "w") as config_file:
        config.write(config_file)


def remove_artibute(path, section, setting, value):  # добавление или изменние параметров в конфигах
    config = configparser.ConfigParser(delimiters='=')
    config.read(path)
    config.remove_option(section, setting)
    with open(path, "w") as config_file:
        config.write(config_file)

def get_artibute(path, section, setting):  # добавление или изменние параметров в конфигах
    config = configparser.ConfigParser(delimiters='=')
    config.read(path)
    section_ = config[section]
    value = section_.get(setting)
    return value

def get_share_folders(path):
    config = configparser.ConfigParser(delimiters='=')
    config.read(path)
    shares = []
    for section in config.sections():
        if section != "global" and section != "homes" and section !=  "printers" and section != "print$":
            shares.append([section, get_artibute(path, section, "path")])
    return shares

def get_level():
    out = os.popen(
        'grep -E "local[0-6]\.\*|local[0-6]\.debug|local[0-6]\.notice|local[0-6]\.alert" /etc/rsyslog.conf').readlines()
    nums_rsyslog = []
    for line in out:
        line = re.sub("                                                ", " ", line)
        level = line.split(".")
        nums_rsyslog.append(int(re.sub("local", "", level[0])))
    nums_rsyslog = sorted(nums_rsyslog)
    out = os.popen(
        'grep -E "local[0-6]\.\*|local[0-6]\.debug|local[0-6]\.notice|local[0-6]\.alert" /etc/samba/smb.conf').readlines()
    nums_smb = []
    for line in out:
        line = re.sub("                                                ", " ", line)
        level = line.split(".")
        nums_smb.append(int(re.sub("local", "", level[0])))
    nums_smb = sorted(nums_smb)
    max_iter = ""
    if len(nums_rsyslog) > 0:
        max_iter = nums_rsyslog[len(nums_rsyslog) - 1]
    if len(nums_smb) > 0:
        if nums_smb[-1] > max_iter:
            max_iter = nums_smb[-1]
    return max_iter

def selinx_patch_change(path):
    path = re.sub(" ", "(.)", path)
    path = f"\"{path}\""
    return path.encode('utf-8')
def do_changes_in_nsswitch():
    if os.popen("grep -r '^passwd:' /etc/nsswitch.conf | grep 'winbind'").readline():
        pass
    else:
        shadownum = os.popen("grep -n '^shadow:' /etc/nsswitch.conf | awk -F ':' '{print $1}'").readline().strip()
        os.system(f"sed -i '{shadownum}s|$| winbind|' /etc/nsswitch.conf")

        passwdnum = os.popen("grep -n '^passwd:' /etc/nsswitch.conf | awk -F ':' '{print $1}'").readline().strip()
        os.system(f"sed -i '{passwdnum}s|$| winbind|' /etc/nsswitch.conf")

        groupnum = os.popen("grep -n '^group:' /etc/nsswitch.conf | awk -F ':' '{print $1}'").readline().strip()
        os.system(f"sed -i '{groupnum}s|$| winbind|' /etc/nsswitch.conf")

        os.system("systemctl enable smb nmb winbind")
        os.system("systemctl restart smb nmb winbind")

class InfoClass(Gtk.Window):
    def __init__(self):
        Gtk.Window.__init__(self)
        self.set_border_width(0)

    def print_info(self, info):
        text = _("Information")
        dialog = Gtk.MessageDialog(parent=self, flags=0, message_type=Gtk.MessageType.INFO,
                                   buttons=Gtk.ButtonsType.OK, text=text)
        dialog.format_secondary_text(info)
        dialog.run()
        dialog.destroy()

    def ticket_dialog(self):
        text = _("Adding a kerberos ticket")
        text2= _("No ticket in cache")
        dialogWindow = Gtk.MessageDialog(parent=self,
                                         modal=True, destroy_with_parent=True,
                                         message_type=Gtk.MessageType.QUESTION,
                                         buttons=Gtk.ButtonsType.OK_CANCEL, text=text2)
        dialogWindow.set_title(text)
        dialogWindow.set_border_width(5)
        dialogBox = dialogWindow.get_content_area()
        labelUser = Gtk.Label(_("Type domain username"))
        dialogBox.pack_start(labelUser, False, False, 0)
        userEntry = Gtk.Entry()
        dialogBox.pack_start(userEntry, False, False, 0)
        labelPass = Gtk.Label(_("Type domain username's password"))
        dialogBox.pack_start(labelPass, False, False, 0)
        pinPass = Gtk.Entry()
        pinPass.set_visibility(False)
        dialogBox.pack_start(pinPass, False, False, 0)
        dialogWindow.show_all()
        response = dialogWindow.run()
        if response == Gtk.ResponseType.OK:
            user = userEntry.get_text()
            passw = pinPass.get_text()
            kinit_out = subprocess.Popen([f"kinit {user} <<< {passw}"], stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True)
            dialogWindow.destroy()
            output, error = kinit_out.communicate()
            if kinit_out.returncode != 0:
                return [False, error.decode("utf-8")]
            return [True]
        else:
            dialogWindow.destroy()
            return [False]


    def print_error(self, error, dbus_mode=False):
        text = _("Error")
        if "No Kerberos credentials" in error:
            text = _("No Kerberos credentials available")
            error = _("Execute commands:\nsu -\nkinit <b>USERNAME</b>\n\nwhere <b>USERNAME</b> is Your's domain one\n\nFor example:\nkinit <b>admin</b>\n\n") + error if not dbus_mode else \
                _("Execute command:\nkinit <b>USERNAME</b>\n\nwhere <b>USERNAME</b> is Your's domain one\n\nFor example:\nkinit <b>admin</b>\n\n") + error

        if "Ticket expired" in error:
            text = _("Ticket expired")
            error = _("Execute commands:\nsu -\nkinit <b>USERNAME</b>\n\nwhere <b>USERNAME</b> is Your's domain one\n\nFor example:\nkinit <b>admin</b>\n\n") + error if not dbus_mode else \
                _("Execute command:\nkinit <b>USERNAME</b>\n\nwhere <b>USERNAME</b> is Your's domain one\n\nFor example:\nkinit <b>admin</b>\n\n") + error
        dialog = Gtk.MessageDialog(parent=self, flags=2, message_type=Gtk.MessageType.ERROR,
                                   buttons=Gtk.ButtonsType.CANCEL, text=text)

        # dialog.format_secondary_text(error)
        dialog.set_markup(error)
        dialog.run()
        dialog.destroy()


    def on_cell_toggled_liststore(self, widget, path, index):
        self.view.set_cursor(path)
        model, iter = self.view.get_selection().get_selected()
        model[iter][index] = not model[iter][index]
        for i in range(0, len(self.liststore)):
            if self.liststore[i][0] == model[iter][0]:
                self.liststore[i][index] = model[iter][index]



    def call_dialog(self, liststore, Title, ColumnText1, TreeViewText, TreeViewText2="",TreeViewText3="",
                    width=0, height=0):
        self.liststore = liststore
        dialogWindow = Gtk.MessageDialog(parent=self,
                                         modal=True, destroy_with_parent=True,
                                         message_type=Gtk.MessageType.QUESTION,
                                         buttons=Gtk.ButtonsType.OK)
        dialogWindow.set_title(Title)
        dialogWindow.set_resizable(True)
        dialogBox = dialogWindow.get_content_area()

        self.clear_liststore = self.liststore
        self.liststore_filter = self.liststore

        self.label = Gtk.Label(_("Type for search"))
        dialogBox.pack_start(self.label, False, False, 0)

        self.entry = Gtk.Entry()
        self.entry.connect("changed", self.refresh_filter)
        dialogBox.pack_start(self.entry, False, False, 0)
        self.widgets = []

        self.filter = self.liststore.filter_new()
        self.filter.set_visible_func(self.visible_names)
        self.view = Gtk.TreeView(model=self.filter)

        renderer_text = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn(ColumnText1, renderer_text, text=0)
        self.view.append_column(column_text)

        renderer_toggle = Gtk.CellRendererToggle()
        renderer_toggle.set_radio(True)
        renderer_toggle.connect("toggled", self.on_cell_toggled_liststore, 1)

        button_access = Gtk.CheckButton()
        self.widgets.append([button_access, 1])

        column_toggle = Gtk.TreeViewColumn(TreeViewText, renderer_toggle, active=1)
        column_toggle.set_alignment(0.5)
        self.view.append_column(column_toggle)

        if TreeViewText2 != "":
            renderer_read_toggle = Gtk.CellRendererToggle()
            renderer_read_toggle.set_radio(True)
            renderer_read_toggle.connect("toggled", self.on_cell_toggled_liststore, 2)

            renderer_write_toggle = Gtk.CellRendererToggle()
            renderer_write_toggle.set_radio(True)
            renderer_write_toggle.connect("toggled", self.on_cell_toggled_liststore, 3)

            column_toggle = Gtk.TreeViewColumn(TreeViewText2, renderer_read_toggle, active=2)
            column_toggle.set_alignment(0.5)
            self.view.append_column(column_toggle)

            column_toggle = Gtk.TreeViewColumn(TreeViewText3, renderer_write_toggle, active=3)
            column_toggle.set_alignment(0.5)
            self.view.append_column(column_toggle)

        scrolled_tree = Gtk.ScrolledWindow()
        scrolled_tree.add(self.view)
        dialogWindow.set_size_request(width, height)
        scrolled_tree.set_size_request(width, height)
        dialogBox.pack_end(scrolled_tree, True, True, 0)
        dialogWindow.show_all()
        response = dialogWindow.run()
        dialogWindow.destroy()
        if response == Gtk.ResponseType.CANCEL:
            self.liststore = liststore

    def refresh_filter(self, widget):
        self.filter.refilter()


    def visible_names(self, model, iter, data=None):
        search_query = self.entry.get_text().lower()
        if search_query == "":
            return True
        value = model.get_value(iter, 0).lower()
        if value.startswith(search_query):
            return True
        else:
            return False

    def choose_folder_dialog(self, widget):
        title = _("Choose file(s)")
        directory = _("Choose directory")
        dialog = Gtk.FileChooserDialog(title=title, parent=self,
                                       action=Gtk.FileChooserAction.SELECT_FOLDER)
        dialog.add_buttons(Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
                           directory, Gtk.ResponseType.OK)
        dialog.set_default_size(800, 400)
        dialog.set_current_folder("/")
        response = dialog.run()
        if response == Gtk.ResponseType.OK:
            file_name = dialog.get_filename()
            dialog.destroy()
            return file_name
        else:
            dialog.destroy()
            return ""

    def on_types_combo_changed(self, combo):
        self.exit_message = combo.get_active_text()
