import gettext
import locale
import os, gi, re, configparser, subprocess

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk, Gdk

t = gettext.translation('share_directory', "/usr/share/locale/")
t.install()
_ = t.gettext

builder = Gtk.Builder()
builder.set_translation_domain("share_directory")
LAYOUT_STYLE_LEGACY, LAYOUT_STYLE_NEW = range(2)

builder.add_from_file('/usr/share/share_directory/ui/share_directory.glade')
# builder_ad.add_from_file('ui/share_directory.glade')
from share_directory.common import *
# from common import *

class IpaGUI:
    def __init__(self):
        self.window = builder.get_object("main_window")
        self.window.set_title(_("Share dir in IPA domain"))
        self.stack = builder.get_object("MainStack")
        self.stack.set_visible_child_name("ipa_page")
        image = Gtk.Image.new_from_file("/usr/share/icons/hicolor/64x64/apps/share-directory.svg")
        image.set_pixel_size(32)
        self.window.set_icon(image.get_pixbuf())
        self.users_list = Gtk.ListStore(str, bool, bool, bool, bool)
        self.groups_list = Gtk.ListStore(str, bool, bool, bool, bool)

        self.get_domain_users_and_groups_list()
        self.entry_folder_path = builder.get_object("entry_folder_ipa_path")
        self.button_choose_folder = builder.get_object("button_choose_folder_ipa")
        self.button_choose_folder.connect("clicked", self.on_button_choose_folder_clicked)

        self.entry_group = builder.get_object("entry_groups_ipa")

        self.button_groups_list = builder.get_object("button_groups_ipa_list")
        self.button_groups_list.connect("clicked", self.on_button_groups_list_clicked)

        self.entry_user = builder.get_object("entry_users_ipa")

        self.button_users_list = builder.get_object("button_users_ipa_list")
        self.button_users_list.connect("clicked", self.on_button_users_list_clicked)
        ###############################################
        self.button_read = builder.get_object("button_read_ipa")
        self.button_write = builder.get_object("button_write_ipa")
        self.button_execute = builder.get_object("button_execute_ipa")

        self.button_recursive = builder.get_object("button_recursive_ipa")
        self.button_recursive.set_active(True)

        ###############################################
        self.button_apply_settings = builder.get_object("button_apply_settings_ipa")
        self.button_apply_settings.connect("clicked", self.on_button_apply_settings_clicked)

        ####################################################
        ####################################################

        self.log_actions_available_list = Gtk.ListStore(str, bool)
        self.log_actions_available_unsuccess_list = Gtk.ListStore(str, bool)
        self.log_prefixes_available_list = Gtk.ListStore(str, bool)

        global log_actions_available, log_prefixes_available
        for i in range(0, len(log_actions_available)):
            self.log_actions_available_list.append(log_actions_available[i])
            self.log_actions_available_unsuccess_list.append(log_actions_available[i])

        for i in range(0, len(log_prefixes_available)):
            self.log_prefixes_available_list.append(log_prefixes_available[i])

        self.entry_share_folder_path = builder.get_object("entry_share_folder_path_ipa")
        self.button_choose_share_folder = builder.get_object("button_choose_share_ipa_folder")
        self.button_choose_share_folder.connect("clicked", self.on_button_choose_share_folder_clicked)

        self.entry_success = builder.get_object("entry_success_ipa")
        self.button_log_actions_available_list = builder.get_object("button_log_actions_available_list_ipa")
        self.button_log_actions_available_list.connect("clicked",
                                                       self.on_button_log_actions_available_list_clicked)
        self.entry_unsuccess = builder.get_object("entry_unsuccess_ipa")

        self.button_log_actions_unsuccess_available_list = builder.get_object(
            "button_log_actions_unsuccess_available_list_ipa")
        self.button_log_actions_unsuccess_available_list.connect("clicked",
                                                                 self.on_button_log_actions_unsuccess_available_list_clicked)
        self.entry_prefix = builder.get_object("entry_prefix_ipa")

        self.button_prefix_list = builder.get_object("button_prefix_list_ipa")
        self.button_prefix_list.connect("clicked", self.on_button_button_prefix_list_clicked)
        ###############################################

        # priorities = [
        #     "debug",
        #     "notice",
        #     "alert"
        # ]
        self.protocols_combo = builder.get_object("protocols_combo_ipa")

        # self.protocols_combo.set_active(0)

        ###############################################

        self.button_apply_log_settings = builder.get_object("button_apply_log_settings_ipa")
        self.button_apply_log_settings.connect("clicked", self.on_button_apply_log_settings_clicked)

        ####################################################
        ####################################################

        self.entry_path_to_remove = builder.get_object("entry_path_to_remove_ipa")

        self.entry_group2 = builder.get_object("entry_group_remove_ipa")
        self.entry_user2 = builder.get_object("entry_user_remove_ipa")
        self.button_all = builder.get_object("button_all_ipa")
        self.button_apply_remove_acl = builder.get_object("button_apply_remove_ipa")
        self.button_apply_remove_acl.connect("clicked", self.on_button_apply_remove_clicked)

        self.entry_path_to_show_acl = builder.get_object("entry_path_to_show_rules_ipa")
        self.button_choose_folder3 = builder.get_object("button_choose_folder_show_rules_ipa")
        self.button_choose_folder3.connect("clicked", self.on_button_choose_folder2_clicked)

        self.button_show_acl = builder.get_object("button_show_acl_ipa")
        self.button_show_acl.connect("clicked", self.on_button_show_acl_clicked)
        self.textview = builder.get_object("textview_ipa")
        self.textbuffer = self.textview.get_buffer()
        self.textview.show()

        self.folders_ipa_treeview = builder.get_object("folders_ipa_treeview")
        self.folders_ipa_list = Gtk.ListStore(str, str)

        folders = get_share_folders("/etc/samba/smb.conf")
        for folder in folders:
            self.folders_ipa_list.append(folder)
        self.folders_ipa_treeview.set_model(self.folders_ipa_list)

        renderer_text = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn(_("Shared folders"), renderer_text, text=0)
        self.folders_ipa_treeview.append_column(column_text)
        self.folders_ipa_treeview.get_selection().connect("changed", self.select_folder)
        self.folders_ipa_treeview.connect('button-press-event', self.on_treeview_doubleclicked)



        self.button_load_ipa = builder.get_object("load_ipa")
        self.button_load_ipa.connect("clicked", self.on_button_load_clicked)

        self.button_remove_ipa = builder.get_object("remove_ipa")
        self.button_remove_ipa.connect("clicked", self.on_button_remove_clicked)

        # selinux to permissive
        # os.system('setenforce 0 && sed -i "s/SELINUX=enforcing/SELINUX=permissive/" /etc/selinux/config')
        if not os.popen("systemctl list-unit-files | grep enabled | grep smb").readlines():
            os.system("systemctl enable smb")
        self.window.show_all()

    def on_treeview_doubleclicked(self, view, event):
        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS and event.button == 1:
            (self.model, self.iter) = view.get_selection().get_selected()
            if self.model and self.iter:
                path = self.model.get_value(self.iter, 1)[0:-1]
                self.entry_folder_path.set_text(path)
                self.entry_share_folder_path.set_text(path)
                self.entry_path_to_remove.set_text(path)
                self.entry_path_to_show_acl.set_text(path)
                self.make_lists(path)

    def select_folder(self, selection):
        (self.model, self.iter) = selection.get_selected()
        self.folder_ipa_selection = selection

    def on_button_load_clicked(self, button):
        if hasattr(self, 'model') and hasattr(self, 'iter'):
            if self.model and self.iter:
                path = self.model.get_value(self.iter, 1)[0:-1]
                self.entry_folder_path.set_text(path)
                self.entry_share_folder_path.set_text(path)
                self.entry_path_to_remove.set_text(path)
                self.entry_path_to_show_acl.set_text(path)
                self.make_lists(path)
                self.folder_ipa_selection.unselect_all()

    def on_button_remove_clicked(self, button):
        try:
            (model, iter) = self.folder_ipa_selection.get_selected()
            counter_home = 0
            for section in model:
                if "/home/" in section[1]:
                    counter_home += 1
            if counter_home == 1:
                os.system("setsebool samba_enable_home_dirs 0")
                home_path = model.get_value(iter, 1).split("/")[1:]
                home_path = f"/home/{home_path[1]}"
                os.system(f"setfacl -b \"{home_path}\"")
            remove_old_section("/etc/samba/smb.conf", model.get_value(iter, 0))
            os.system(f"setfacl -b \"{str(model.get_value(iter, 1)[0:-1])}\"")
            os.system("systemctl restart smb")
            self.folders_ipa_list.remove(iter)
            self.folder_ipa_selection.unselect_all()
        except Exception as e:
            print(e)

    def on_button_choose_share_folder_clicked(self, button):
        win = InfoClass()
        self.entry_share_folder_path.set_text(win.choose_folder_dialog(self))

    def on_button_choose_folder_clicked(self, button):
        win = InfoClass()
        self.entry_folder_path.set_text(win.choose_folder_dialog(self))

    def on_button_choose_folder2_clicked(self, button):
        win = InfoClass()
        self.entry_path_to_show_acl.set_text(win.choose_folder_dialog(self))

    def get_domain_users_and_groups_list(self):
        self.make_lists()

    def on_button_log_actions_available_list_clicked(self, button):
        win = InfoClass()
        log_actions = _("Log actions")
        selected_actions = _("Selected actions")
        if win.call_dialog(self.log_actions_available_list, log_actions,
                           log_actions, selected_actions, "", "", 390, 300):
            text = ""
            for i in range(0, len(self.log_actions_available_list)):
                if self.log_actions_available_list[i][1]:
                    text += f"{self.log_actions_available_list[i][0]}," if i < len(self.log_actions_available_list) - 1 \
                        else f"{self.log_actions_available_list[i][0]}"
            if len(text) != 0:
                if text[-1] == ",":
                    text = text[:-1]
            else:
                text = "none"
            self.entry_success.set_text(text)

    def on_button_log_actions_unsuccess_available_list_clicked(self, button):
        win = InfoClass()
        log_actions = _("Log actions")
        selected_actions = _("Selected actions")
        if win.call_dialog(self.log_actions_available_unsuccess_list, log_actions,
                           log_actions, selected_actions, "", "", 390, 300):
            text = ""
            for i in range(0, len(self.log_actions_available_unsuccess_list)):
                if self.log_actions_available_unsuccess_list[i][1]:
                    text += f"{self.log_actions_available_unsuccess_list[i][0]}," if i < len(
                        self.log_actions_available_unsuccess_list) - 1 else f"{self.log_actions_available_unsuccess_list[i][0]}"
            if len(text) != 0:
                if text[-1] == ",":
                    text = text[:-1]
            else:
                text = "none"
            self.entry_unsuccess.set_text(text)

    def on_button_button_prefix_list_clicked(self, button):
        win = InfoClass()
        log_prefixes = _("Log prefixes")
        selected_prefixes = _("Selected prefixes")
        if win.call_dialog(self.log_prefixes_available_list, log_prefixes,
                           log_prefixes, selected_prefixes, "", "", 390, 300):
            text = ""
            for i in range(0, len(self.log_prefixes_available_list)):
                if self.log_prefixes_available_list[i][1]:
                    text += f"{self.log_prefixes_available_list[i][0]}," if i < len(self.log_prefixes_available_list) - \
                                                                            1 else f"{self.log_prefixes_available_list[i][0]}"
            if len(text) != 0:
                if text[-1] == ",":
                    text = text[:-1]
            self.entry_prefix.set_text(text)

    def make_lists(self, path=""):
        LANG = os.popen("echo $LANG").readlines()[0].strip()
        self.users_list_allowed = []
        self.groups_list_allowed = []
        self.users_list.clear()
        self.groups_list.clear()
        if "ru" in LANG:
            temp_users_list = os.popen(
                """sed s'|Имя учётной записи пользователя: ||'g <<< $(ipa user-find --all | grep "Имя учётной записи пользователя: ")""").readlines()
            temp_groups_list = os.popen(
                """sed s'|Имя группы: ||'g <<< $(sed s'|ID группы: ||'g <<< $(ipa group-find """
                """--all | grep -E "Имя группы: *|*ID группы:*"))""").readlines()
        else:
            temp_users_list = os.popen(
                """sed s'|User login: ||'g <<< $(LANG=C ipa user-find --all | grep "User login: ")""").readlines()
            temp_groups_list = os.popen("""sed s'|Group name: ||'g <<< $(sed s'|GID: ||'g <<<"""
                                        """$(LANG=C ipa group-find --all | grep -E "Group name: *|*GID:*"))""").readlines()
        if path != "":
            acls = subprocess.Popen(f'getfacl -p \"{path}\"',
                                    stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                                    shell=True)

            acls_out = acls.communicate()[0].decode("utf-8").strip()
            users = re.findall(r'^user\:\S+\:.{3}', acls_out, re.MULTILINE + re.DOTALL)
            groups = re.findall(r'^group\:\S+\:.{3}', acls_out, re.MULTILINE + re.DOTALL)

            for i in range(0, len(temp_users_list)):
                flag_allowed_user = False
                flag_read_user = False
                flag_write_user = False
                flag_execute_user = True
                name = temp_users_list[i].strip()
                for uzver in users:
                    uzver = uzver.split(":")
                    if name in re.sub("\\\\040", " ", uzver[1]).lower():
                        flag_allowed_user = True
                        if "r" in uzver[2]:
                            flag_read_user = True
                        if "w" in uzver[2]:
                            flag_write_user = True
                        self.users_list_allowed.append(
                            [name, flag_allowed_user, flag_read_user, flag_write_user, flag_execute_user])
                self.users_list.append([name, flag_allowed_user, flag_read_user, flag_write_user, flag_execute_user])

            leng = len(temp_groups_list)
            for i in range(0, leng):
                if i < leng - 1:
                    if ((str(temp_groups_list[i].strip()).isalnum() or str(temp_groups_list[i].strip()).isalpha())
                            and
                            str(temp_groups_list[i + 1].strip()).isnumeric()):
                        name = re.sub("\w*\\\\", "", temp_groups_list[i].strip())
                        flag_allowed_group = False
                        flag_read_group = False
                        flag_write_group = False
                        flag_execute_group = False
                        for gzver in groups:
                            gzver = gzver.split(":")
                            if name in re.sub("\\\\040", " ", gzver[1]).lower():
                                flag_allowed_group = True
                                if "r" in gzver[2]:
                                    flag_read_group = True
                                if "w" in gzver[2]:
                                    flag_write_group = True
                                if "x" in gzver[2]:
                                    flag_execute_group = True
                                self.groups_list_allowed.append([name, flag_allowed_group, flag_read_group,
                                                                 flag_write_group, flag_execute_group])
                        self.groups_list.append([temp_groups_list[i].strip(), flag_allowed_group, flag_read_group,
                                                 flag_write_group, flag_execute_group])
        else:
            for i in range(0, len(temp_users_list)):
                self.users_list.append([temp_users_list[i].strip(), False, False, False, False])

            leng = len(temp_groups_list)
            for i in range(0, leng):
                if i < leng - 1:
                    if ((str(temp_groups_list[i].strip()).isalnum() or str(temp_groups_list[i].strip()).isalpha())
                            and
                            str(temp_groups_list[i + 1].strip()).isnumeric()):
                        self.groups_list.append([temp_groups_list[i].strip(), False, False, False, False])

    def on_button_users_list_clicked(self, button):
        win = InfoClass()
        domain_users = _("Domain users")
        access_allowed = _("Access allowed")
        win.call_dialog(self.users_list, domain_users, domain_users, access_allowed, _("Read"), _("Write"),
                        600, 500)
        self.users_list_allowed = []
        for user in self.users_list:
            if user[1]:
                self.users_list_allowed.append(user)

    def on_button_groups_list_clicked(self, button):
        win = InfoClass()
        domain_groups = _("Domain groups")
        access_allowed = _("Access allowed")
        win.call_dialog(self.groups_list, domain_groups, domain_groups, access_allowed, _("Read"), _("Write"),
                        700, 500)
        self.groups_list_allowed = []
        for group in self.groups_list:
            if group[1]:
                self.groups_list_allowed.append(group)

    def on_button_apply_settings_clicked(self, button):
        self.create_folder()
        if self.check_keytabs():
            if self.add_rules():
                domain = get_ipa_domain_name()

                netbios = domain.split(".")[0]

                valid_users = ""
                read_list = ""
                write_list = ""

                # [name, include, r, w, x]
                if len(self.groups_list_allowed) > 0:
                    for i in range(0, len(self.groups_list_allowed)):
                        if self.groups_list_allowed[i][3]:
                            write_list += f"@{self.groups_list_allowed[i][0]}, "
                        if self.groups_list_allowed[i][2]:
                            read_list += f"@{self.groups_list_allowed[i][0]}, "
                        valid_users += f"@{self.groups_list_allowed[i][0]}, "

                if len(self.users_list_allowed) > 0:
                    for i in range(0, len(self.users_list_allowed)):
                        print(self.users_list_allowed[i][3])
                        if self.users_list_allowed[i][3]:
                            write_list += f"{self.users_list_allowed[i][0]}, "
                        if self.users_list_allowed[i][2]:
                            read_list += f"{self.users_list_allowed[i][0]}, "
                        valid_users += f"{self.users_list_allowed[i][0]}, "


                if valid_users != "":
                    valid_users = valid_users[0:-2]
                if write_list != "":
                    write_list = write_list[0:-2]
                if read_list != "":
                    read_list = read_list[0:-2]

                print(valid_users, write_list, read_list)
                self.write_to_smb(self.entry_folder_path.get_text(), netbios,
                                  domain, valid_users, read_list, write_list)
                self.selinux_configure()
                self.net_groupmap()
                self.krb5()
                win = InfoClass()
                self.entry_share_folder_path.set_text(self.entry_folder_path.get_text())
                self.entry_path_to_remove.set_text(self.entry_folder_path.get_text())
                self.entry_path_to_show_acl.set_text(self.entry_folder_path.get_text())
                win.print_info(_('Folder setup completed successfully'))
        else:
            win = InfoClass()
            win.print_info('keytab с сервера IPA не найден')

    def check_keytabs(self):
        if not os.path.exists("/etc/samba/samba.keytab"):
            return False
        return True

    def krb5(self):
        os.system(
            "sed -i 's;default_ccache_name = KEYRING:persistent:%{uid};default_ccache_name = FILE:/tmp/krb5cc_%{uid};g' /etc/krb5.conf")

    def on_button_apply_log_settings_clicked(self, button):
        win = InfoClass()
        prefix = self.entry_prefix.get_text()
        prefix = re.sub("%", "%%", prefix)
        success = self.entry_success.get_text()
        unsuccess = self.entry_unsuccess.get_text()
        path = self.entry_share_folder_path.get_text()
        priority = self.protocols_combo.get_active_text()
        local_number = get_level()
        mas = path.split("/")
        folder = mas[len(mas) - 1]
        if get_section("/etc/samba/smb.conf", folder):
            if prefix != "" and success != "" and unsuccess != "" and priority != "":

                grep_level = os.popen(f'grep "{folder}.log" /etc/rsyslog.conf').readlines()
                if len(grep_level) > 0:
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="vfs objects",
                                 value="full_audit")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:prefix",
                                 value=f"{prefix}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:failure",
                                 value=f"{unsuccess}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:success",
                                 value=f"{success}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:facility",
                                 value=f"local{local_number}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:priority",
                                 value=f"{priority}")
                    # opening the file in read mode
                    file = open("/etc/rsyslog.conf", "r")
                    replacement = ""
                    # using the for loop
                    for line in file:
                        line = line.strip()
                        if f"{folder}.log" in line:
                            changes = f"local{local_number}.{priority}                                                " \
                                      f"-/var/log/samba/{folder}.log"
                        else:
                            changes = line
                        replacement = replacement + changes + "\n"

                    file.close()
                    if not os.path.exists(f"/var/log/samba/{folder}.log"):
                        os.system(f"touch /var/log/samba/{folder}.log")
                    # opening the file in write mode
                    file = open("/etc/rsyslog.conf", "w")
                    file.write(replacement)
                    file.close()
                    os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
                    os.system("systemctl restart smb rsyslog")
                    win.print_info(_('Folder setup completed successfully.\n'
                                     'Please reboot for the settings to take effect.'))
                elif local_number == "":
                    local_number = 0
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="vfs objects",
                                 value="full_audit")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:prefix",
                                 value=f"{prefix}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:failure",
                                 value=f"{unsuccess}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:success",
                                 value=f"{success}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:facility",
                                 value=f"local{local_number + 1}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:priority",
                                 value=f"{priority}")
                    os.system(
                        f"echo 'local{local_number + 1}.{priority}                                                "
                        f"-/var/log/samba/{folder}.log' >> /etc/rsyslog.conf")
                    if not os.path.exists(f"/var/log/samba/{folder}.log"):
                        os.system(f"touch /var/log/samba/{folder}.log")
                    os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
                    os.system("systemctl restart smb rsyslog")

                    win.print_info(_('Folder setup completed successfully.\n'
                                     'Please reboot for the settings to take effect.'))
                elif 0 <= local_number <= 5:  # до local5, тк local5+1 = local6 т.е. уже максимум,
                    # local7 - занят системой, local8+ уже не поддерживается
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="vfs objects",
                                 value="full_audit")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:prefix",
                                 value=f"{prefix}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:failure",
                                 value=f"{unsuccess}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:success",
                                 value=f"{success}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:facility",
                                 value=f"local{local_number + 1}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:priority",
                                 value=f"{priority}")
                    os.system(
                        f"echo 'local{local_number + 1}.{priority}                                                "
                        f"-/var/log/samba{folder}.log' >> /etc/rsyslog.conf")
                    os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
                    if not os.path.exists(f"/var/log/samba/{folder}.log"):
                        os.system(f"touch /var/log/samba/{folder}.log")
                    os.system("systemctl restart smb rsyslog")
                    win.print_info(_('Folder setup completed successfully.\n'
                                     'Please reboot for the settings to take effect.'))
                else:
                    win.print_error(_('A maximum of 7 network folders are available for logging.\n'
                                      'You need to disable logging for the previous network folder.\n'
                                      'Files: /etc/samba/smb.conf and /etc/rsyslog.conf'))

    def selinux_configure(self):
        output = os.popen("getenforce").readlines()[0]
        output = output.strip()
        path = self.entry_folder_path.get_text()
        win = InfoClass()
        if output == "Enforcing":
            if path != "":
                if not os.popen(f"semanage fcontext --list \"{path}\" | grep samba_share_t").readlines():
                    os.system(f"semanage fcontext -a -t samba_share_t {selinx_patch_change(path)}")
                    os.system(f"restorecon -R -v {selinx_patch_change(path)}")
                if "off" in os.popen("getsebool samba_export_all_rw").readline():
                    os.system("setsebool -P samba_export_all_rw 1")
                if "/home/" in path:
                    if "off" in os.popen("getsebool samba_enable_home_dirs").readline():
                        os.system("setsebool -P samba_enable_home_dirs 1")
            else:
                win.print_info(_('Type path'))

    def on_button_apply_remove_acl_clicked(self, button):
        if self.button_all.get_active():
            if self.remove_all_rules():
                win = InfoClass()
                win.print_info(_("Rules deleted successfully"))
        else:
            if self.remove_rules():
                win = InfoClass()
                win.print_info(_("Rules deleted successfully"))

    def on_button_show_acl_clicked(self, button):
        self.get_rules()

    def create_folder(self):
        path = self.entry_folder_path.get_text()
        path_to = path
        win = InfoClass()
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_folder_path.set_text(path_to)
            if not os.path.exists(f"{str(path_to)}"):
                os.makedirs(f'{str(path_to)}')     
        else:
            win.print_info(_('Type path'))

    def try_remove_atr(self, atrs_list, option, name, folder):
        if f"{name}, " in atrs_list:
            atrs_list = re.sub(f"{name}, ", "", atrs_list)
            set_artibute('/etc/samba/smb.conf', folder, option, atrs_list)
        if f", {name}" in atrs_list:
            atrs_list = re.sub(f", {name}", "", atrs_list)
            set_artibute('/etc/samba/smb.conf', folder, option, atrs_list)

    def remove_rules(self):
        path = self.entry_path_to_remove.get_text()
        path_to = path
        name = self.entry_user2.get_text()
        group = self.entry_group2.get_text()
        win = InfoClass()
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_path_to_remove.set_text(path_to)
            if not os.path.exists(path_to):
                win.print_error("The path does not exist")
                return False
            else:
                if name == "" and group == "":
                    win.print_info(_('Specify whose rights you want to remove'))
                    return False
                else:
                    if name != "":
                        strk = f'setfacl -x u:"{name}" \"{path}\"'
                        output = os.system(strk)
                        if output != 0:
                            win.print_error(_('An error occurred while deleting permissions, '
                                              'please check that the username is correct'))
                            return False
                        strk = f'setfacl -x d:u:"{name}" \"{path}\"'
                        os.system(strk)
                        output = os.system(strk)
                        if output != 0:
                            win.print_error(_('An error occurred while deleting permissions, '
                                              'please check that the username is correct'))
                            return False

                        folder = path.split("/")[-1]

                        atrs_valide = get_artibute('/etc/samba/smb.conf', folder, 'valid users')
                        atrs_read = get_artibute('/etc/samba/smb.conf', folder, 'read list')
                        atrs_write = get_artibute('/etc/samba/smb.conf', folder, 'write list')

                        self.try_remove_atr(atrs_list=atrs_valide, option='valid users', name=f"{name}",
                                            folder=folder)
                        self.try_remove_atr(atrs_list=atrs_read, option='read list', name=f"{name}",
                                            folder=folder)
                        self.try_remove_atr(atrs_list=atrs_write, option='write list', name=f"{name}",
                                            folder=folder)

                    if group != "":
                        strk = f'setfacl -x g:"{group}" \"{path}\"'
                        os.system(strk)
                        output = os.system(strk)
                        if output != 0:
                            win.print_error(_('An error occurred while deleting permissions, '
                                              'please check that the groupname is correct'))
                            return False
                        strk = f'setfacl -x d:g:"{group}" \"{path}\"'
                        os.system(strk)
                        output = os.system(strk)
                        if output != 0:
                            win.print_error(_('An error occurred while deleting permissions, '
                                              'please check that the groupname is correct'))
                            return False

                        folder = path.split("/")[-1]
                        atrs_valide = get_artibute('/etc/samba/smb.conf', folder, 'valid users')
                        atrs_read = get_artibute('/etc/samba/smb.conf', folder, 'read list')
                        atrs_write = get_artibute('/etc/samba/smb.conf', folder, 'write list')

                        self.try_remove_atr(atrs_list=atrs_valide, option='valid users', name=f"@{group}",
                                            folder=folder)
                        self.try_remove_atr(atrs_list=atrs_read, option='read list', name=f"@{group}",
                                            folder=folder)
                        self.try_remove_atr(atrs_list=atrs_write, option='write list', name=f"@{group}",
                                            folder=folder)

                    return True
        else:
            win.print_info(_('Type path'))
            return False

    def remove_all_rules(self):
        path = self.entry_path_to_remove.get_text()
        path_to = path
        win = InfoClass()
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_path_to_remove.set_text(path_to)
            if os.path.exists(path_to):
                strk = f"setfacl -b \"{str(path_to)}\""
                os.system(strk)
                return True
            else:
                win.print_error("The path does not exist")
                return False
        else:
            win.print_info(_('Type path'))
            return False

    def net_groupmap(self):
        os.system("net -s /dev/null groupmap add sid=S-1-5-32-546 unixgroup=nobody type=builtin")
        os.system("systemctl restart smb")

    def write_to_smb(self, path, netbios, domain, valid_users, read_list, write_list):
        mas = path.split("/")
        folder = mas[len(mas) - 1]
        os.system("cp /etc/samba/smb.conf /etc/samba/smb.conf.$(date '+%F-%T')")
        set_artibute('/etc/samba/smb.conf', 'global', "workgroup", netbios.upper())  # workgroup
        set_artibute('/etc/samba/smb.conf', 'global', "realm", domain.upper())  # realm
        set_artibute('/etc/samba/smb.conf', 'global', "dedicated keytab file", "FILE:/etc/samba/samba.keytab")
        set_artibute('/etc/samba/smb.conf', 'global', "kerberos method", "dedicated keytab")
        set_artibute('/etc/samba/smb.conf', 'global', "log file", "/var/log/samba/log.%%m")
        set_artibute('/etc/samba/smb.conf', 'global', "security", "ads")

        if not get_section("/etc/samba/smb.conf", folder):
            add_new_section('/etc/samba/smb.conf', folder)
            if path[len(path) - 1] != "\\":
                set_artibute('/etc/samba/smb.conf', folder, "path", f"{path}/")
            else:
                set_artibute('/etc/samba/smb.conf', folder, "path", path)
            set_artibute('/etc/samba/smb.conf', folder, "writable", "no")

            if valid_users != "":
                set_artibute('/etc/samba/smb.conf', folder, "valid users", valid_users)
            if read_list != "":
                set_artibute('/etc/samba/smb.conf', folder, "read list", read_list)
            if write_list != "":
                set_artibute('/etc/samba/smb.conf', folder, "write list", write_list)

            set_artibute('/etc/samba/smb.conf', folder, "guest ok", "no")
            set_artibute('/etc/samba/smb.conf', folder, "inherit acls", "yes")
            set_artibute('/etc/samba/smb.conf', folder, "create mask", "0660")
            set_artibute('/etc/samba/smb.conf', folder, "directory mask", "0770")

            set_artibute('/etc/samba/smb.conf', folder, "browseable", "yes")
        else:
            if path[len(path) - 1] != "\\":
                set_artibute('/etc/samba/smb.conf', folder, "path", f"{path}/")
            else:
                set_artibute('/etc/samba/smb.conf', folder, "path", path)
            set_artibute('/etc/samba/smb.conf', folder, "writable", "no")

            if valid_users != "":
                set_artibute('/etc/samba/smb.conf', folder, "valid users", valid_users)
            else:
                try:
                    remove_artibute('/etc/samba/smb.conf', folder, "valid users")
                except Exception as e:
                    pass
            if read_list != "":
                set_artibute('/etc/samba/smb.conf', folder, "read list", read_list)
            else:
                try:
                    remove_artibute('/etc/samba/smb.conf', folder, "read users")
                except Exception as e:
                    pass
            if write_list != "":
                set_artibute('/etc/samba/smb.conf', folder, "write list", write_list)
            else:
                try:
                    remove_artibute('/etc/samba/smb.conf', folder, "write users")
                except Exception as e:
                    pass
            set_artibute('/etc/samba/smb.conf', folder, "guest ok", "no")
            set_artibute('/etc/samba/smb.conf', folder, "inherit acls", "yes")
            set_artibute('/etc/samba/smb.conf', folder, "create mask", "0660")
            set_artibute('/etc/samba/smb.conf', folder, "directory mask", "0770")

            set_artibute('/etc/samba/smb.conf', folder, "browseable", "yes")
            os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
        flag_exists = False
        for fol in self.folders_ipa_list:
            if folder == fol[0]:
                flag_exists = True
        if not flag_exists:
            self.folders_ipa_list.append([folder, path])


    def on_button_apply_remove_clicked(self, button):
        if self.button_all.get_active():
            if self.remove_all_rules():
                win = InfoClass()
                win.print_info(_('Rules deleting completed successfully.\n'
                                 'Please reboot for the settings to take effect.'))
        else:
            if self.remove_rules():
                win = InfoClass()
                win.print_info(_('Rules deleting completed successfully.\n'
                                 'to completely remove rights, select the "Remove all rights" checkbox'))

    def add_rules(self):
        path = self.entry_folder_path.get_text()
        win = InfoClass()

        if len(self.users_list_allowed) == 0 and len(self.groups_list_allowed) == 0:
            win.print_info(_('Specify to whom you want to assign rights'))
        else:
            strk = f"setfacl -b \"{str(path)}\""
            os.system(strk)
            recursive = ""
            if self.button_recursive.get_active():
                recursive = "-R"
            domain = get_domain_name()
            path_root = path.split("/")[1:]

            if len(self.groups_list_allowed) > 0 and path != "":
                strk = f'chown {recursive} root:root {path} && chmod 2770 "{path}" '
                # groups = group.split(",")
                for group_member in self.groups_list_allowed:
                    group = group_member[0]
                    mas = ["-", "-", "-"]
                    if group_member[2]:
                        mas[0] = "r"
                    if group_member[3]:
                        mas[1] = "w"
                    if group_member[4]:
                        mas[2] = "x"
                    permissions = "".join([str(p) for p in mas])

                    if len(path_root) == 1:
                        strk += f'&& setfacl {recursive} -m default:group:"{str(domain)}\\{group}":{permissions} "{path}" ' \
                                f'&& setfacl {recursive} -m group:"{str(domain)}\\{group}":{permissions} "{path}" '
                    else:
                        strk += f'&& setfacl {recursive} -m default:group:"{str(domain)}\\{group}":{permissions} "{path}" ' \
                                f'&& setfacl {recursive} -m group:"{str(domain)}\\{group}":{permissions} "{path}" '
                        temp_path = ""
                        for i in range(0, len(path_root) - 1):
                            temp_path += f" \"/{'/'.join(path_root[0:i + 1])}\""
                        strk += f'&& setfacl -m group:"{str(domain)}\\{group}":{permissions} {temp_path} '

                print("group change\n##########", strk)
                output = os.system(f"{strk}")
                if output != 0:
                    win.print_error(_("An error occurred while applying rights, "
                                      "check that the user and group names are correct"))
                    return False
            if len(self.users_list_allowed) > 0 and path != "":
                strk = f'chown {recursive} root:root {path} && chmod 2770 "{path}" '
                for user_member in self.users_list_allowed:
                    user = user_member[0]
                    mas = ["-", "-", "-"]
                    if user_member[2]:
                        mas[0] = "r"
                    if user_member[3]:
                        mas[1] = "w"
                    if user_member[4]:
                        mas[2] = "x"
                    permissions = "".join([str(p) for p in mas])
                    if len(path_root) == 1:
                        strk += f'&& setfacl {recursive} -m default:user:"{str(domain)}\\{user}":{permissions} "{path}" ' \
                                f'&& setfacl {recursive} -m user:"{str(domain)}\\{user}":{permissions} "{path}" '
                    else:
                        strk += f'&& setfacl {recursive} -m default:user:"{str(domain)}\\{user}":{permissions} "{path}" ' \
                                f'&& setfacl {recursive} -m user:"{str(domain)}\\{user}":{permissions} "{path}" '
                        temp_path = ""
                        for i in range(0, len(path_root) - 1):
                            temp_path += f" \"/{'/'.join(path_root[0:i + 1])}\""
                        strk += f'&& setfacl -m group:"{str(domain)}\\{user}":{permissions} {temp_path} '
                print("name change\n##########", strk)
                output = os.system(f"{strk}")
                if output != 0:
                    win.print_error(_("An error occurred while applying rights, "
                                      "check that the user and group names are correct"))
                    return False
            return True

    def get_rules(self):
        win = InfoClass()
        path = self.entry_path_to_show_acl.get_text()
        path_to = path
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_path_to_show_acl.set_text(path_to)
            if os.path.exists(path_to):
                end_text = os.popen(f"getfacl -p  \"{path}\"").readlines()
                strk = "".join([str(k) for k in end_text])
                self.textbuffer.set_text(str(strk))
            else:
                win.print_error("The path does not exist")
