import gettext
import locale
import os, gi, re, configparser, subprocess

gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
from gi.repository import Gtk, Gdk

t = gettext.translation('share_directory', "/usr/share/locale/")
t.install()
_ = t.gettext

builder = Gtk.Builder()
builder.set_translation_domain("share_directory")
LAYOUT_STYLE_LEGACY, LAYOUT_STYLE_NEW = range(2)

builder.add_from_file('/usr/share/share_directory/ui/share_directory.glade')
# builder_ad.add_from_file('ui/share_directory.glade')
from share_directory.common import *
# from common import *

class LocalGUI:
    def __init__(self):
        self.window = builder.get_object("main_window")
        self.window.set_title(_("Share dir in workgroup"))
        self.stack = builder.get_object("MainStack")
        self.stack.set_visible_child_name("local_page")
        image = Gtk.Image.new_from_file("/usr/share/icons/hicolor/64x64/apps/share-directory.svg")
        image.set_pixel_size(32)
        self.window.set_icon(image.get_pixbuf())
        self.users_list = Gtk.ListStore(str, bool)
        self.get_local_users_list()


        self.entry_folder_local_path = builder.get_object("entry_folder_local_path")

        self.button_choose_folder_local = builder.get_object("button_choose_folder_local")
        self.button_choose_folder_local.connect("clicked", self.on_button_choose_folder_local_clicked)

        self.entry_workgroup_name = builder.get_object("entry_workgroup_name")
        try:
            wg = get_artibute("/etc/samba/smb.conf", "global", "workgroup")
            if wg != "":
                self.entry_workgroup_name.set_text(wg)
        except Exception as e:
            print(e)
        self.entry_netbios_name = builder.get_object("entry_netbios_name")

        hostname = os.popen("hostname").readlines()
        hostname = hostname[0].split(".")[0].strip()
        self.entry_netbios_name.set_text(hostname)

        self.folders_local_treeview = builder.get_object("folders_local_treeview")

        self.entry_smbpasswd = builder.get_object("entry_smbpasswd")
        self.entry_smbpasswd.set_visibility(False)

        self.switch = builder.get_object("switch")
        self.switch.connect("notify::active", self.on_switch_activated)
        self.switch.set_active(True)

        self.entry_users_local = builder.get_object("entry_users_local")

        self.button_users_local_list = builder.get_object("button_users_local_list")
        self.button_users_local_list.connect("clicked", self.on_button_users_local_list_clicked)

        self.button_read_local = builder.get_object("button_read_local")
        self.button_write_local = builder.get_object("button_write_local")
        # self.button_execute_local = builder_ad.get_object("button_execute_local")
        self.button_anonymous_local = builder.get_object("button_anonymous_local")

        self.button_anonymous_local .connect("toggled", self.on_anon_toggled)


        self.button_apply_settings_local = builder.get_object("button_apply_settings_local")
        self.button_apply_settings_local.connect("clicked", self.on_button_apply_settings_local_clicked)

        self.log_actions_available_list = Gtk.ListStore(str, bool)
        self.log_actions_available_unsuccess_list = Gtk.ListStore(str, bool)
        self.log_prefixes_available_list = Gtk.ListStore(str, bool)

        global log_actions_available, log_prefixes_available
        for i in range(0, len(log_actions_available)):
            self.log_actions_available_list.append(log_actions_available[i])
            self.log_actions_available_unsuccess_list.append(log_actions_available[i])

        for i in range(0, len(log_prefixes_available)):
            self.log_prefixes_available_list.append(log_prefixes_available[i])
        ########################
        ########################

        self.entry_share_folder_path_local = builder.get_object("entry_share_folder_path_local")

        self.button_choose_share_local_folder = builder.get_object("button_choose_share_local_folder")
        self.button_choose_share_local_folder.connect("clicked", self.on_button_choose_share_local_folder_clicked)

        self.entry_success = builder.get_object("entry_success_local")

        self.button_log_actions_available_list = builder.get_object("button_log_actions_available_list_local")
        self.button_log_actions_available_list.connect("clicked",
                                                       self.on_button_log_actions_available_list_clicked)
        self.entry_unsuccess = builder.get_object("entry_unsuccess_local")

        self.button_log_actions_unsuccess_available_list = builder.get_object(
            "button_log_actions_unsuccess_available_list_local")
        self.button_log_actions_unsuccess_available_list.connect("clicked",
                                                                 self.on_button_log_actions_unsuccess_available_list_clicked)
        self.entry_prefix = builder.get_object("entry_prefix")

        self.button_prefix_list = builder.get_object("button_prefix_list")
        self.button_prefix_list.connect("clicked", self.on_button_prefix_list_clicked)

        self.protocols_combo = builder.get_object("protocols_combo")

        self.protocols_combo.set_active(0)

        self.button_apply_log_settings_local = builder.get_object("button_apply_log_settings_local")
        self.button_apply_log_settings_local.connect("clicked", self.on_button_apply_log_settings_local_clicked)
        ########################
        ########################

        self.entry_path_to_remove_local = builder.get_object("entry_path_to_remove_local")
        self.button_choose_folder_local_remove = builder.get_object("button_choose_folder_local_remove")
        self.button_choose_folder_local_remove.connect("clicked", self.on_button_choose_folder_local_remove_clicked)

        self.entry_user_remove_local = builder.get_object("entry_user_remove_local")

        self.button_all_local = builder.get_object("button_all_local")

        self.button_apply_remove_local = builder.get_object("button_apply_remove_local")
        self.button_apply_remove_local.connect("clicked", self.on_button_apply_remove_local_clicked)

        ########################
        ########################

        self.entry_path_to_show_rules_local = builder.get_object("entry_path_to_show_rules_local")

        self.button_choose_folder_show_rules_local = builder.get_object("button_choose_folder_show_rules_local")
        self.button_choose_folder_show_rules_local.connect("clicked",
                                                           self.on_button_choose_folder_show_rules_local_clicked)

        self.button_show_acl_local = builder.get_object("button_show_acl_local")
        self.button_show_acl_local.connect("clicked", self.on_button_show_acl_local_clicked)

        self.textview_local = builder.get_object("textview_local")
        self.textbuffer = self.textview_local.get_buffer()
        self.textview_local.show()

        self.folders_local_list = Gtk.ListStore(str, str)
        folders = get_share_folders("/etc/samba/smb.conf")
        for folder in folders:
            self.folders_local_list.append(folder)
        self.folders_local_treeview.set_model(self.folders_local_list)

        renderer_text = Gtk.CellRendererText()
        column_text = Gtk.TreeViewColumn(_("Shared folders"), renderer_text, text=0)
        self.folders_local_treeview.append_column(column_text)
        self.folders_local_treeview.get_selection().connect("changed", self.select_folder)
        self.folders_local_treeview.connect('button-press-event', self.on_treeview_doubleclicked)


        self.button_load_local = builder.get_object("load_local")
        self.button_load_local.connect("clicked", self.on_button_load_clicked)

        self.button_remove_local = builder.get_object("remove_local")
        self.button_remove_local.connect("clicked", self.on_button_remove_clicked)

        if not os.popen("systemctl list-unit-files | grep enabled | grep smb").readlines():
            os.system("systemctl enable smb")
        self.window.show_all()

    def on_treeview_doubleclicked(self, view, event):
        if event.type == Gdk.EventType.DOUBLE_BUTTON_PRESS and event.button == 1:
            (self.model, self.iter) = view.get_selection().get_selected()
            if self.model and self.iter:
                path = self.model.get_value(self.iter, 1)[0:-1]
                self.entry_folder_local_path.set_text(path)
                self.entry_share_folder_path_local.set_text(path)
                self.entry_path_to_remove_local.set_text(path)
                self.entry_path_to_show_rules_local.set_text(path)

                self.get_local_users_list()
                anon = get_artibute("/etc/samba/smb.conf", self.model.get_value(self.iter, 0), "guest ok")
                if anon == "yes":
                    self.button_anonymous_local.set_active(True)
                    self.entry_smbpasswd.set_sensitive(False)
                    self.switch.set_sensitive(False)
                else:
                    mask = get_artibute("/etc/samba/smb.conf", self.model.get_value(self.iter, 0), "force create mode")
                    if mask == "0770":
                        self.button_read_local.set_active(True)
                        self.button_write_local.set_active(True)
                    elif mask == "0550":
                        self.button_read_local.set_active(True)
                    elif mask == "0330":
                        self.button_write_local.set_active(True)
                    self.button_anonymous_local.set_active(False)

    def on_anon_toggled(self, button):
        if button.get_active():
            self.button_users_local_list.set_sensitive(False)
            self.entry_smbpasswd.set_sensitive(False)
            self.switch.set_sensitive(False)

            # self.button_read_local.set_sensitive(False)
            # self.button_write_local.set_sensitive(False)
        else:
            self.button_users_local_list.set_sensitive(True)
            self.entry_smbpasswd.set_sensitive(True)
            self.switch.set_sensitive(True)
            # self.button_read_local.set_sensitive(True)
            # self.button_write_local.set_sensitive(True)

    def select_folder(self, selection):
        (self.model, self.iter) = selection.get_selected()
        self.folder_local_selection = selection

    def on_button_load_clicked(self, button):
        if hasattr(self, 'model') and hasattr(self, 'iter'):
            if self.model and self.iter:
                path = self.model.get_value(self.iter, 1)[0:-1]
                self.entry_folder_local_path.set_text(path)
                self.entry_share_folder_path_local.set_text(path)
                self.entry_path_to_remove_local.set_text(path)
                self.entry_path_to_show_rules_local.set_text(path)

                self.get_local_users_list()
                anon = get_artibute("/etc/samba/smb.conf", self.model.get_value(self.iter, 0), "guest ok")
                if anon == "yes":
                    self.button_anonymous_local.set_active(True)
                else:
                    mask = get_artibute("/etc/samba/smb.conf", self.model.get_value(self.iter, 0), "force create mode")
                    if mask == "0770":
                        self.button_read_local.set_active(True)
                        self.button_write_local.set_active(True)
                    elif mask == "0550":
                        self.button_read_local.set_active(True)
                    elif mask == "0330":
                        self.button_write_local.set_active(True)
                self.folder_local_selection.unselect_all()

    def on_button_remove_clicked(self, button):
        try:
            (model, iter) = self.folder_local_selection.get_selected()
            counter_home = 0
            for section in model:
                if "/home/" in section[1]:
                    counter_home += 1
            if counter_home == 1:
                os.system("setsebool samba_enable_home_dirs 0")
                home_path = model.get_value(iter, 1).split("/")[1:]
                home_path = f"/home/{home_path[1]}"
                os.system(f"setfacl -b \"{home_path}\"")
            remove_old_section("/etc/samba/smb.conf", model.get_value(iter, 0))
            os.system("systemctl restart smb")
            self.folders_local_list.remove(iter)
            self.folder_local_selection.unselect_all()
        except Exception as e:
            print(e)

    def on_switch_activated(self, switch, gparam):
        if not switch.get_state():
            self.entry_smbpasswd.set_visibility(True)
        else:
            self.entry_smbpasswd.set_visibility(False)

    def on_button_choose_folder_local_clicked(self, button):
        win = InfoClass()
        self.entry_folder_local_path.set_text(win.choose_folder_dialog(self))

    def on_button_choose_share_local_folder_clicked(self, button):
        win = InfoClass()
        self.entry_share_folder_path_local.set_text(win.choose_folder_dialog(self))


    def on_button_choose_folder_local_remove_clicked(self, button):
        win = InfoClass()
        self.entry_path_to_remove_local.set_text(win.choose_folder_dialog(self))

    def on_button_choose_folder_show_rules_local_clicked(self, button):
        win = InfoClass()
        self.entry_path_to_show_rules_local.set_text(win.choose_folder_dialog(self))

    def get_local_users_list(self):
        self.users_list.clear()
        self.users_allowed = []
        temp_users_list = os.popen("cut -d: -f1,3 /etc/passwd | egrep ':[0-9]{4}$' | cut -d: -f1").readlines()
        wg_users = os.popen("getent group smb_share").readline()
        if wg_users != "" and wg_users is not None:
            wg_users = wg_users.split(":")[-1].split(",")
            for i in range(0, len(temp_users_list)):
                name = temp_users_list[i].strip()
                flag_wg = False
                for wg_user in wg_users:
                    if name == wg_user.strip():
                        flag_wg = True
                        self.users_allowed.append(name)
                self.users_list.append([name, flag_wg])
        else:
            for i in range(0, len(temp_users_list)):
                self.users_list.append([temp_users_list[i].strip(), False])

    def on_button_users_local_list_clicked(self, button):
        win = InfoClass()
        workgroup_users = _("Workgroup users")
        access_allowed = _("Access allowed")
        win.call_dialog(self.users_list, workgroup_users, workgroup_users,
                        access_allowed, "", "",  450, 300)
        text = ""
        self.users_allowed = []
        for i in range(0, len(self.users_list)):
            if self.users_list[i][1]:
                self.users_allowed.append(self.users_list[i][0])
            # self.entry_users_local.set_text(text)

    def on_button_log_actions_available_list_clicked(self, button):
        win = InfoClass()
        log_actions = _("Log actions")
        selected_actions = _("Selected actions")
        if win.call_dialog(self.log_actions_available_list, log_actions,
                           log_actions, selected_actions, "", "",  420, 300):
            text = ""
            for i in range(0, len(self.log_actions_available_list)):
                if self.log_actions_available_list[i][1]:
                    text += f"{self.log_actions_available_list[i][0]}," if i < len(self.log_actions_available_list) - 1 \
                        else f"{self.log_actions_available_list[i][0]}"
            if len(text) != 0:
                if text[-1] == ",":
                    text = text[:-1]
            else:
                text = "none"
            self.entry_success.set_text(text)

    def on_button_log_actions_unsuccess_available_list_clicked(self, button):
        win = InfoClass()
        log_actions = _("Log actions")
        selected_actions = _("Selected actions")
        if win.call_dialog(self.log_actions_available_unsuccess_list, log_actions,
                           log_actions, selected_actions, "", "",  420, 300):
            text = ""
            for i in range(0, len(self.log_actions_available_unsuccess_list)):
                if self.log_actions_available_unsuccess_list[i][1]:
                    text += f"{self.log_actions_available_unsuccess_list[i][0]}," if i < len(
                        self.log_actions_available_unsuccess_list) - 1 else f"{self.log_actions_available_unsuccess_list[i][0]}"
            if len(text) != 0:
                if text[-1] == ",":
                    text = text[:-1]
            else:
                text = "none"
            self.entry_unsuccess.set_text(text)

    def on_button_prefix_list_clicked(self, button):
        win = InfoClass()
        log_prefixes = _("Log prefixes")
        selected_prefixes = _("Selected prefixes")
        if win.call_dialog(self.log_prefixes_available_list, log_prefixes,
                           log_prefixes, selected_prefixes, "", "",  420, 300):
            text = ""
            for i in range(0, len(self.log_prefixes_available_list)):
                if self.log_prefixes_available_list[i][1]:
                    text += f"{self.log_prefixes_available_list[i][0]}," if i < len(self.log_prefixes_available_list) - \
                                                                            1 else f"{self.log_prefixes_available_list[i][0]}"
            if len(text) != 0:
                if text[-1] == ",":
                    text = text[:-1]
            self.entry_prefix.set_text(text)

    def on_button_apply_log_settings_local_clicked(self, button):
        prefix = self.entry_prefix.get_text()
        prefix = re.sub("%", "%%", prefix)
        success = self.entry_success.get_text()
        unsuccess = self.entry_unsuccess.get_text()
        path = self.entry_share_folder_path_local.get_text()
        priority = self.protocols_combo.get_active_text()
        local_number = get_level()
        win = InfoClass()
        mas = path.split("/")
        folder = mas[len(mas) - 1]
        if get_section("/etc/samba/smb.conf", folder):
            if prefix != "" and success != "" and unsuccess != "" and priority != "":

                grep_level = os.popen(f'grep "{folder}.log" /etc/rsyslog.conf').readlines()
                if len(grep_level) > 0:
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="vfs objects",
                                 value="full_audit")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:prefix",
                                 value=f"{prefix}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:failure",
                                 value=f"{unsuccess}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:success",
                                 value=f"{success}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:facility",
                                 value=f"local{local_number}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:priority",
                                 value=f"{priority}")
                    # opening the file in read mode
                    file = open("/etc/rsyslog.conf", "r")
                    replacement = ""
                    # using the for loop
                    for line in file:
                        line = line.strip()
                        if f"{folder}.log" in line:
                            changes = f"local{local_number}.{priority}                                                " \
                                      f"-/var/log/samba/{folder}.log"
                        else:
                            changes = line
                        replacement = replacement + changes + "\n"

                    file.close()
                    if not os.path.exists(f"/var/log/samba/{folder}.log"):
                        os.system(f"touch /var/log/samba/{folder}.log")
                    # opening the file in write mode
                    file = open("/etc/rsyslog.conf", "w")
                    file.write(replacement)
                    file.close()
                    os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
                    os.system("systemctl restart smb rsyslog")
                    win.print_info(_('Folder setup completed successfully.\n'
                                     'Please reboot for the settings to take effect.'))
                elif local_number == "":
                    local_number = 0
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="vfs objects",
                                 value="full_audit")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:prefix",
                                 value=f"{prefix}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:failure",
                                 value=f"{unsuccess}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:success",
                                 value=f"{success}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:facility",
                                 value=f"local{local_number + 1}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:priority",
                                 value=f"{priority}")
                    os.system(
                        f"echo 'local{local_number + 1}.{priority}                                                "
                        f"-/var/log/samba/{folder}.log' >> /etc/rsyslog.conf")
                    if not os.path.exists(f"/var/log/samba/{folder}.log"):
                        os.system(f"touch /var/log/samba/{folder}.log")
                    os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
                    os.system("systemctl restart smb rsyslog")

                    win.print_info(_('Folder setup completed successfully.\n'
                                     'Please reboot for the settings to take effect.'))
                elif 0 <= local_number <= 5:  # до local5, тк local5+1 = local6 т.е. уже максимум,
                    # local7 - занят системой, local8+ уже не поддерживается
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="vfs objects",
                                 value="full_audit")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:prefix",
                                 value=f"{prefix}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:failure",
                                 value=f"{unsuccess}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:success",
                                 value=f"{success}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:facility",
                                 value=f"local{local_number + 1}")
                    set_artibute(path="/etc/samba/smb.conf", section=folder, setting="full_audit:priority",
                                 value=f"{priority}")
                    os.system(
                        f"echo 'local{local_number + 1}.{priority}                                                "
                        f"-/var/log/samba/{folder}.log' >> /etc/rsyslog.conf")
                    os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
                    if not os.path.exists(f"/var/log/samba/{folder}.log"):
                        os.system(f"touch /var/log/samba/{folder}.log")
                    os.system("systemctl restart smb rsyslog")
                    win.print_info(_('Folder setup completed successfully.\n'
                                     'Please reboot for the settings to take effect.'))
                else:
                    win.print_error(_('A maximum of 7 network folders are available for logging.\n'
                                      'You need to disable logging for the previous network folder.\n'
                                      'Files: /etc/samba/smb.conf and /etc/rsyslog.conf'))

    def on_button_apply_settings_local_clicked(self, button):
        if self.create_folder():
            atrs = self.add_rules()
            if atrs:
                self.write_to_smb(self.entry_folder_local_path.get_text(), atrs)
                self.selinux_configure()
                self.entry_share_folder_path_local.set_text(self.entry_folder_local_path.get_text())
                self.entry_path_to_remove_local.set_text(self.entry_folder_local_path.get_text())
                self.entry_path_to_show_rules_local.set_text(self.entry_folder_local_path.get_text())
                win = InfoClass()
                win.print_info(_('Folder setup completed successfully.\n'
                                 'Please reboot for the settings to take effect.'))

    def selinux_configure(self):
        output = os.popen("getenforce").readlines()[0]
        output = output.strip()
        path = self.entry_folder_local_path.get_text()
        win = InfoClass()
        if output == "Enforcing":
            if path != "":
                if not os.popen(f"semanage fcontext --list \"{path}\" | grep samba_share_t").readlines():
                    os.system(f"semanage fcontext -a -t samba_share_t {selinx_patch_change(path)}")
                    os.system(f"restorecon -R -v {selinx_patch_change(path)}")
                if "off" in os.popen("getsebool samba_export_all_rw").readline():
                    os.system("setsebool -P samba_export_all_rw 1")
                if "/home/" in path:
                    if "off" in os.popen("getsebool samba_enable_home_dirs").readline():
                        os.system("setsebool -P samba_enable_home_dirs 1")
            else:
                win.print_info(_('Type path'))

    def on_button_apply_remove_local_clicked(self, button):
        if self.button_all_local.get_active():
            if self.remove_all_rules():
                win = InfoClass()
                win.print_info(_('Rules deleting completed successfully.\n'
                                 'Please reboot for the settings to take effect.'))
        else:
            if self.remove_rules():
                win = InfoClass()
                win.print_info(_('Rules deleting completed successfully.\n'
                                 'to completely remove rights, select the "Remove all rights" checkbox'))

    def on_button_show_acl_local_clicked(self, button):
        self.get_rules()

    def create_folder(self):
        path = self.entry_folder_local_path.get_text()
        path_to = path
        win = InfoClass()
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_folder_local_path.set_text(path_to)
            if not os.path.exists(f"{str(path_to)}"):
                os.makedirs(f'{str(path_to)}')    
            return True
        else:
            win.print_info(_('Type path'))
            return False

    def remove_rules(self):
        path = self.entry_path_to_remove_local.get_text()
        path_to = path
        name = self.entry_user_remove_local.get_text()
        win = InfoClass()
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_path_to_remove_local.set_text(path_to)
            if not os.path.exists(path_to):
                win.print_error("The path does not exist")
                return False
            else:
                if name == "":
                    win.print_info(_('Specify whose rights you want to remove'))
                    return False
                else:
                    if name != "":
                        grep_username = os.popen(f"cat /etc/group | grep smb_share | grep {name}").readlines()
                        if grep_username:
                            strk = f"gpasswd -d {name} smb_share"
                            output = os.system(strk)
                            if output != 0:
                                win.print_error(_('An error occurred while deleting permissions, '
                                                  'please check that the username is correct'))
                                return False
                            return True
        else:
            win.print_info(_('Type path'))
            return False

    def remove_all_rules(self):
        path = self.entry_path_to_remove_local.get_text()
        path_to = path
        win = InfoClass()
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_path_to_remove_local.set_text(path_to)
            if path[len(path) - 1] != "/":
                path_to = path + "/"
                self.entry_path_to_remove_local.set_text(path_to)
            if os.path.exists(path_to):
                config = configparser.ConfigParser(delimiters='=')
                with open('/etc/samba/smb.conf', 'r') as smb:
                    config.read_file(smb)
                section = path.split("/")
                section = section[len(section) - 1]
                if section in config.sections():
                    config.remove_section(section)
                    with open("/etc/samba/smb.conf", "w") as f:
                        config.write(f)
                    os.system("groupdel smb_share && groupadd smb_share && systemctl restart smb")
                    return True
                else:
                    win.print_error('This folder setting was not found,\n'
                                    'available for configured network folders.')
            else:
                win.print_error("The path does not exist")
                return False
        else:
            win.print_info(_('Type path'))
            return False

    def write_to_smb(self, path, atrs):
        mas = path.split("/")
        folder = mas[len(mas) - 1]
        mask = "0" + atrs[0]
        # atrs [mask, anonymous, netbios, workgroup]
        os.system("cp /etc/samba/smb.conf /etc/samba/smb.conf.$(date '+%F-%T')")
        set_artibute('/etc/samba/smb.conf', 'global', "workgroup", atrs[3])
        set_artibute('/etc/samba/smb.conf', 'global', "server string", "Samba Server %%v")
        set_artibute('/etc/samba/smb.conf', 'global', "netbios name", atrs[2])
        set_artibute('/etc/samba/smb.conf', 'global', "security", "user")
        set_artibute('/etc/samba/smb.conf', 'global', "passdb backend", "tdbsam")

        set_artibute('/etc/samba/smb.conf', 'global', "log file", "/var/log/samba/log.%%m")
        set_artibute('/etc/samba/smb.conf', 'global', "max log size", "50")
        os.system("sed -i 's|%%|%|g' /etc/samba/smb.conf")
        if not get_section("/etc/samba/smb.conf", folder):
            add_new_section('/etc/samba/smb.conf', folder)
        if path[len(path) - 1] != "\\":
            set_artibute('/etc/samba/smb.conf', folder, "path", f"{path}/")
        else:
            set_artibute('/etc/samba/smb.conf', folder, "path", path)
        set_artibute('/etc/samba/smb.conf', folder, "browsable", "yes")
        set_artibute('/etc/samba/smb.conf', folder, "writable", "yes")

        if atrs[1]:  # anonymous option
            set_artibute('/etc/samba/smb.conf', 'global', "map to guest", "bad user")
            set_artibute('/etc/samba/smb.conf', folder, "guest ok", "yes")
        else:
            set_artibute('/etc/samba/smb.conf', folder, "guest ok", "no")
            set_artibute('/etc/samba/smb.conf', folder, "valid users", "@smb_share")
            set_artibute('/etc/samba/smb.conf', folder, "force directory mode", mask)
            set_artibute('/etc/samba/smb.conf', folder, "force create mode", mask)
            set_artibute('/etc/samba/smb.conf', folder, "vfs objects", "full_audit")
        os.system("systemctl restart smb")
        flag_exists = False
        for fol in self.folders_local_list:
            if folder == fol[0]:
                flag_exists = True
        if not flag_exists:
            self.folders_local_list.append([folder, path])

    def add_rules(self):
        os.system("getent group smb_share || groupadd smb_share")
        path = self.entry_folder_local_path.get_text()
        # names = re.sub('[.:;]', ',', self.entry_users_local.get_text())
        win = InfoClass()
        mask = ""
        anonymous = ""
        if len(self.users_allowed) == 0 and not self.button_anonymous_local.get_active():
            win.print_info(_('Specify to whom you want to assign rights'))
            return False
        else:
            mas = ["-", "-", "x"]

            if self.button_read_local.get_active() and self.button_write_local.get_active():
                mask = "770"
            elif self.button_write_local.get_active():
                mask = "330"
            elif self.button_read_local.get_active():
                mask = "550"
            else:
                mask = "00"
            if self.button_anonymous_local.get_active():
                anonymous = True
            else:
                anonymous = False
                password = self.entry_smbpasswd.get_text()
            netbios = self.entry_netbios_name.get_text()
            workgroup = self.entry_workgroup_name.get_text()
            if workgroup == "":
                self.entry_workgroup_name.set_text("WORKGROUP")
                workgroup == "WORKGROUP"
            strk = ""
            if anonymous and path != "":
                strk = f'chown -R nobody:nobody "{path}" && chmod -R {mask} "{path}"'
            elif path != "" and password != "" and netbios != "":
                if self.button_read_local.get_active():
                    mas[0] = "r"
                if self.button_write_local.get_active():
                    mas[1] = "w"
                strk = f'chown -R :smb_share "{path}" && chmod -R {mask} "{path}" '
                for user in self.users_allowed:
                    grep_username = os.popen(f"cat /etc/group | grep smb_share | grep {user}").readlines()
                    if grep_username:
                        pass
                    else:
                        strk += f"&&  usermod -a -G smb_share {user} && echo -ne '{password}\n{password}\n' | smbpasswd -a -s {user} "
                    if "/home/" in path:
                        permissions = "".join([str(p) for p in mas])
                        strk += f"&& setfacl -R -m default:group:\"smb_share\":\"{permissions}\" \"{path}\" " \
                                f"&& setfacl -R -m group:\"smb_share\":\"{permissions}\" \"{path}\" "
            else:
                win.print_error(_("Check that all fields are filled"))
                return False
            output = os.system(f"{strk}")
            if output != 0:
                win.print_error(_("An error occurred while applying rights, "
                                  "check that the user names are correct"))
                return False
            return [mask, anonymous, netbios, workgroup]

    def get_rules(self):
        win = InfoClass()
        path = self.entry_path_to_show_rules_local.get_text()
        path_to = path
        if path != "":
            if path[0] != "/":
                path_to = "/" + path
                self.entry_path_to_show_rules_local.set_text(path_to)
            if os.path.exists(path_to):

                config = configparser.ConfigParser(delimiters='=')
                config.read("/etc/samba/smb.conf")
                mas = path.split("/")
                folder = mas[len(mas) - 1]
                strk = f"[{folder}]\n"
                if folder in config.sections():
                    for item in config.items(folder):
                        strk += f"{item[0]} = {item[1]}\n"
                    self.textbuffer.set_text(str(strk))
                else:
                    win.print_error(_('This folder is not on the network'))
            else:
                win.print_error("The path does not exist")
